


IgM_SV = {
	enabled = true,
	list = {},
}


local frame = CreateFrame("Frame")

local list;	-- will be pointed to the relevant realm entry inside IgM_SV.list on init (if IgM is enabled)

local IndexToPlayer = {}
local SysIgnoreList = {}



local strmatch = string.match
local strupper = string.upper
local strsub = string.sub
local tostring,pairs,ipairs,next = tostring,pairs,ipairs,next




-----------------------------------------------------------------------
-- Utilities



local function fixname(name)
	if strmatch(name, "^[a-z]") then		-- can't just strupper first char. flat out. they can be utf-8. and hope that the user typed the rest in correct case (and the first in correct case if it isn't ASCII!)
		return strupper(strsub(name,1,1))..strsub(name,2);
	end
	return name;
end

local function splitargs(str)
	local n,r = strmatch(str, "^ *(..-) *: *(.*)");
	if not n then
		n = strmatch(str, "^ *(..-) *$")
	end
	if not n then
		print("IgnoreMore: '"..str.."' does not look like a valid player name")
		return
	end
	if strmatch(n, " ") and not strmatch(n, "^.[^ ]-%-") then
		print("IgnoreMore: '"..n.."' does not look like a valid player name")
		return
	end
	return n,r
end



-----------------------------------------------------------------------
-- Fake event generators

local function generateEvent(evtname,...)
	local a1,a2,a3,a4,a5,a6,a7,a8,a9 = arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9;
	local e = event;
	local t = this;
	
	-- print("Generating event",evtname,...)
	for i,frame in ipairs({GetFramesRegisteredForEvent(evtname)}) do
		local onevent = frame:GetScript("OnEvent")
		if onevent then
			-- print("    '"..(frame:GetName() or tostring(frame)).."'",tostring(onevent))
			this=frame
			event=evtname
			arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9 = ...
			onevent(frame,evtname,...)
		end
	end
	
	this=t;
	event=e;
	arg1,arg2,arg3,arg4,arg5,arg6,arg7,arg8,arg9 = a1, a2, a3, a4, a5, a6, a7, a8, a9;
end

local function generateSysMsg(msg)
	generateEvent("CHAT_MSG_SYSTEM", msg, "", "", "", "", "", "", "", "", "", "")
end






-----------------------------------------------------------------------
-----------------------------------------------------------------------
--
-- API Hooks
--

local orig_AddIgnore = AddIgnore;
local orig_AddOrDelIgnore = AddOrDelIgnore;
local orig_DelIgnore = DelIgnore;
local orig_GetIgnoreName = GetIgnoreName;
local orig_GetNumIgnores = GetNumIgnores;
local orig_GetSelectedIgnore = GetSelectedIgnore;
local orig_SetSelectedIgnore = SetSelectedIgnore;


-- AddIgnore

function IgM_AddIgnore(name, reason, quiet)
	if type(name)~="string" or strlen(name)<2 then
		return;
	end
	name=fixname(name);
	
	if(name==UnitName("player")) then
		if(not quiet) then
			generateSysMsg(ERR_IGNORE_SELF);
		end
		return false;
	end
	
	if(not reason and list[name] and type(list[name])=="table") then
		if(not quiet) then 
			generateSysMsg(format(ERR_IGNORE_ALREADY_S, name));
		end
		return false;
	end
	
	if(list[name] and type(list[name])=="table") then
		-- just an update, no creation
	else
		tinsert(IndexToPlayer, name);
		list[name]={};
	end
	
	list[name].time=time();
	list[name].reason=reason;
	
	if(not quiet) then
		if(reason) then 
			generateSysMsg(format(ERR_IGNORE_ADDED_S, name)..' "'..reason..'"')
		else
			generateSysMsg(format(ERR_IGNORE_ADDED_S, name))
		end			
	end
	
	generateEvent("IGNORELIST_UPDATE");
	return false;
end

function AddIgnore(name)
	if(not list) then 
		return orig_AddIgnore(name);
	end
	local n,r = splitargs(name)
	if not n then return end
	return IgM_AddIgnore(n, r)
end


-- DelIgnore

function IgM_DelIgnore(name, quiet)
	name=fixname(name);
	
	if(not list[name] or type(list[name])~="table") then
		if(not quiet) then
			generateSysMsg(ERR_IGNORE_NOT_FOUND);
		end
		return false;
	end

	for k,v in pairs(IndexToPlayer) do
		if(v==name) then
			tremove(IndexToPlayer, k);
			break;
		end
	end
	
	list[name]=0;
	
	if(SysIgnoreList[name]) then
		orig_DelIgnore(name);
		SysIgnoreList[name]=nil;
	end
	
	if(not quiet) then
		generateSysMsg(format(ERR_IGNORE_REMOVED_S,name));
		generateEvent("IGNORELIST_UPDATE");
	end
	return true;
end

function DelIgnore(name)
	if(not list) then 
		return orig_DelIgnore(name);
	end
	IgM_DelIgnore(name);
end


-- AddOrDelIgnore

function IgM_AddOrDelIgnore(name, reason, quiet)
	name=fixname(name);
	if(list[name] and type(list[name])=="table") then
		IgM_DelIgnore(name,quiet);
	else
		IgM_AddIgnore(name,reason,quiet);
	end
end

function AddOrDelIgnore(name)
	if(not list) then 
		return orig_AddOrDelIgnore(name);
	end
	local n,r = splitargs(name)
	if not n then return end
	if r then
		return IgM_AddIgnore(n, r);
	else
		return IgM_AddOrDelIgnore(name);
	end
end


-- GetIgnoreName

function GetIgnoreName(i)
	if(not list) then
		return orig_GetIgnoreName(i);
	end
	return IndexToPlayer[i] or "Unknown";
end


-- GetNumIgnores

function GetNumIgnores()
	if(not list) then
		return orig_GetNumIgnores();
	end
	return getn(IndexToPlayer);
end


-- GetSelectedIgnore   (god this is pointless..)

function GetSelectedIgnore()
	if(not list) then
		return orig_GetSelectedIgnore();
	end
	return IgM_SelectedIgnore;
end


-- SetSelectedIgnore   (god this is pointless..)

function SetSelectedIgnore(i)
	if(not list) then
		return orig_SetSelectedIgnore(i);
	end
	IgM_SelectedIgnore = i;
end


-- IgM_GetIgnoreReason
function IgM_GetIgnoreReason(who)
	if(not list) then
		return nil;
	end
	local i = tonumber(who);
	if(i) then
		who=IndexToPlayer[i];
	end
	if(not who) then
		return nil;
	end
	if(not list[who]) then
		return nil;
	end
	return list[who].reason or "";
end






-----------------------------------------------------------------------
-----------------------------------------------------------------------
--
-- UI: Extend IgnoreList with time and reason info
--


local column2 = {};


local orig_IgnoreList_Update = IgnoreList_Update;
function IgnoreList_Update(...)
	
	orig_IgnoreList_Update(...);
	if(not list or not FriendsFrameIgnoreScrollFrame) then
		return;
	end
	
	local ignoreOffset = FauxScrollFrame_GetOffset(FriendsFrameIgnoreScrollFrame);
	local maxwid=0;
	local buttonwid=0;
	for i=1, IGNORES_TO_DISPLAY, 1 do
		local ignoreIndex = i + ignoreOffset;
		local nameText = getglobal("FriendsFrameIgnoreButton"..i.."ButtonTextName");
		local ignoreButton = getglobal("FriendsFrameIgnoreButton"..i);
		if(nameText and ignoreButton) then
			buttonwid = ignoreButton:GetWidth();
			if(not column2[i]) then
				column2[i] = ignoreButton:CreateFontString("FontString", "OVERLAY", "GameFontHighlightSmall");
				column2[i]:SetJustifyH("LEFT");
				column2[i]:SetPoint("BOTTOMRIGHT", ignoreButton);
				column2[i]:SetPoint("TOP", ignoreButton);
			end
			
			local txt="";
			maxwid = max(maxwid, nameText:GetStringWidth());
			local entry = list[nameText:GetText()];
			if(entry and type(entry)=="table") then
				if(entry.time) then 
					if(time() - entry.time < 3600*24*200) then
						txt=txt..date("%d %b", entry.time);
					else
						txt=txt..date("%b %Y", entry.time);
					end
				end
				if(entry.reason) then
					txt=txt..": "..entry.reason;
				end
			end
			column2[i]:SetText(txt);
		end
	end
	
	for i=1, IGNORES_TO_DISPLAY, 1 do
		if(column2[i]) then
			column2[i]:SetWidth(buttonwid-maxwid-30);
		end
	end
	
	
end


local last_FriendsFrameIgnoreButton_OnClick = { time=0 }

local orig_FriendsFrameIgnoreButton_OnClick = FriendsFrameIgnoreButton_OnClick;
function FriendsFrameIgnoreButton_OnClick(...)
	orig_FriendsFrameIgnoreButton_OnClick(...);
	if(not list) then
		return;
	end
	if(last_FriendsFrameIgnoreButton_OnClick.button == this and 
	   GetTime() - last_FriendsFrameIgnoreButton_OnClick.time < 0.3) then
		StaticPopup_Show("EDIT_IGNORE_REASON");
	end
	
	last_FriendsFrameIgnoreButton_OnClick.button = this;
	last_FriendsFrameIgnoreButton_OnClick.time = GetTime();
end


-- Make a popup dialog for editing ignore reasons
local dlg = {}
for k,v in pairs(StaticPopupDialogs["SET_FRIENDNOTE"]) do
	dlg[k] = v;
end
dlg.maxLetters=128
dlg.text = "Reason for ignoring this player:"
dlg.OnAccept = function(self)
	IgM_AddIgnore(GetIgnoreName(GetSelectedIgnore()), self.wideEditBox:GetText());
end
dlg.EditBoxOnEnterPressed = function(self)
	local parent = self:GetParent();
	dlg.OnAccept(parent)
	parent:Hide();
end
dlg.OnShow = function(self)
	self.wideEditBox:SetText(IgM_GetIgnoreReason(GetSelectedIgnore()));
	self.wideEditBox:SetFocus();
end

StaticPopupDialogs["EDIT_IGNORE_REASON"] = dlg;





-----------------------------------------------------------------------
-----------------------------------------------------------------------
--
-- The actual filtering!
--


-----------------------------------------------------------------------
-- Our chat filtering function, installed via ChatFrame_AddMessageEventFilter

local function filterFunc(self,event,msg,source,...)
	-- print("Message: ", msg, "  Source: ", source)
	if list and type(source)=="string" and source~="" and type(list[source])=="table" then
		if event=="CHAT_MSG_WHISPER_INFORM" then
			return false, arg1.."  (NOTE: You are ignoring this player!)", source, ...
		end
		return true
	end
end


-----------------------------------------------------------------------
-- Initialization

frame:Show()
frame:SetScript("OnUpdate", function()
	frame:Hide()

	if(not IgM_SV.enabled) then
		list = nil;
		return;
	end
	
	local realm = GetRealmName() .. "-" .. UnitFactionGroup("player");
	
	if(not IgM_SV.list[realm]) then
		IgM_SV.list[realm] = {}
	end
	
	list = IgM_SV.list[realm];
	
	-- Loop through the system ignore list and import entries to our list. Remember which ones were on the system ignore list! (So we know to remove them if removal is requested)
	for i=(orig_GetNumIgnores()),1,-1 do
		local name = orig_GetIgnoreName(i);
		if type(name)~="string" or name=="" then
			-- arf, wth... (apparently some user has managed to have this happen, i don't know through which mechanism, but ..)
		elseif name==UNKNOWN then
			-- recently-deleted/transferred user showing up temporarily in the system ignore list; skip it
		elseif(not list[name]) then
			-- Ignoremore was not aware of this system entry, so add it to the SV!
			list[name] = {time=time(), reason="From "..UnitName("player").."'s system ignore list"};
			SysIgnoreList[name] = true;
		elseif(type(list[name])~="table") then
			-- This user is no longer ignored. So remove from system ignore list.
			orig_DelIgnore(name)
		else
			SysIgnoreList[name] = true;
		end
	end
	
	-- Make an indexed list of our entries (several of the APIs work by indices)
	for k,v in pairs(list) do
		if type(k)~="string" or k=="" then
			list[k] = nil  -- arf, wth... (apparently some user has managed to have this happen, i don't know through which mechanism, but ..)
		end
		if(type(v)=="table") then
			tinsert(IndexToPlayer, k);
		end
	end
	
	table.sort(IndexToPlayer);	-- sort it by alpha for easier navigation
	
	-- Install our chat filter
	for _,event in pairs({
		"CHAT_MSG_ACHIEVEMENT",
		"CHAT_MSG_BATTLEGROUND",
		"CHAT_MSG_BATTLEGROUND_LEADER",
		"CHAT_MSG_CHANNEL",
		"CHAT_MSG_CHANNEL_JOIN",
		"CHAT_MSG_CHANNEL_LEAVE",
		"CHAT_MSG_EMOTE",
		"CHAT_MSG_GUILD",
		"CHAT_MSG_GUILD_ACHIEVEMENT",
		"CHAT_MSG_OFFICER",
		"CHAT_MSG_PARTY",
		"CHAT_MSG_RAID",
		"CHAT_MSG_RAID_LEADER",
		"CHAT_MSG_RAID_WARNING",
		"CHAT_MSG_SAY",
		"CHAT_MSG_TEXT_EMOTE",
		"CHAT_MSG_WHISPER",
		"CHAT_MSG_WHISPER_INFORM",
		"CHAT_MSG_YELL",
		})
	do
		ChatFrame_AddMessageEventFilter(event, filterFunc)
	end
	
end)


